package de.ozgcloud.nachrichten.postfach.osiv2;

import static org.assertj.core.api.Assertions.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.condition.EnabledIfEnvironmentVariable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.DummyStringBasedIdentifier;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import lombok.SneakyThrows;

@SpringBootTest(classes = TestApplication.class)
@ActiveProfiles({ "stage", "itcase" })
@EnabledIfEnvironmentVariable(named = "SH_STAGE_CLIENT_SECRET", matches = ".+")
class OsiPostfachRemoteServiceRemoteITCase {

	@Autowired
	private OsiPostfachRemoteService osiPostfachRemoteService;

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add(
				"spring.security.oauth2.client.registration.osi2.client-secret",
				() -> System.getenv("SH_STAGE_CLIENT_SECRET")
		);
		registry.add(
				"ozgcloud.osiv2.proxy.host",
				() -> matchProxyRegex(System.getenv("HTTP_PROXY")).group(1)
		);
		registry.add(
				"ozgcloud.osiv2.proxy.port",
				() -> matchProxyRegex(System.getenv("HTTP_PROXY")).group(2)
		);
	}

	private static Matcher matchProxyRegex(String text) {
		var matcher = Pattern.compile("([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+):([0-9]+)").matcher(text);
		if (matcher.find()) {
			return matcher;
		}
		throw new IllegalArgumentException("Proxy host and port not found in '%s'".formatted(text));
	}

	private PostfachNachricht createNachricht() {
		return PostfachNachrichtTestFactory.createBuilder()
				.replyOption(PostfachNachricht.ReplyOption.POSSIBLE)
				.postfachAddress(PostfachAddressTestFactory.createBuilder()
						.identifier(DummyStringBasedIdentifier.builder()
								.mailboxId("49b5a7e2-5e60-4baf-8ccf-1f5b94b570f3")
								.build())
						.build())
				.build();
	}

	@DisplayName("should not fail sending message")
	@Test
	@SneakyThrows
	void shouldNotFailSendingMessage() {
		var nachricht = createNachricht();

		assertThatCode(() -> osiPostfachRemoteService.sendMessage(nachricht))
				.doesNotThrowAnyException();
	}

	@DisplayName("should receive messages")
	@Test
	void shouldReceiveMessages() {
		var messages = osiPostfachRemoteService.getAllMessages().toList();

		assertThat(messages)
				.isNotEmpty()
				.extracting(PostfachNachricht::getMessageId)
				.doesNotContainNull();
	}

	@DisplayName("should delete message")
	@Test
	void shouldDeleteMessage() {
		assertThatCode(() -> osiPostfachRemoteService.deleteMessage("2cec3eac-66d2-4de0-bc6b-652b8e985ceb"))
				.doesNotThrowAnyException();
	}
}
